﻿
'This class was based on the VelocityCacheAdapter clas
'MSDN Magazine Sept 2011 Data Points Column Sample Code
'(Julie Lerman)
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Security.Cryptography
Imports System.Text
Imports EFCachingProvider.Caching
Imports Microsoft.ApplicationServer.Caching

Public Class AppFabricCache
    Implements ICache
    Private _cache As DataCache

    Public Sub New(ByVal cache As DataCache)
        _cache = cache
    End Sub

    Public Function GetItem(ByVal key As String, <System.Runtime.InteropServices.Out()> ByRef value As Object) As Boolean Implements ICache.GetItem
        key = GetCacheKey(key)
        value = _cache.Get(key)

        Return value IsNot Nothing
    End Function

    Public Sub PutItem(ByVal key As String, ByVal value As Object, ByVal dependentEntitySets As IEnumerable(Of String), ByVal slidingExpiration As TimeSpan, ByVal absoluteExpiration As Date) Implements ICache.PutItem
        key = GetCacheKey(key)
        _cache.Put(key, value, absoluteExpiration.Subtract(Date.Now), dependentEntitySets.Select(Function(c) New DataCacheTag(c)).ToList())

        For Each dep In dependentEntitySets
            CreateRegionIfNeeded(dep)
            _cache.Put(key, " ", dep)
        Next dep

    End Sub

    Public Sub InvalidateSets(ByVal entitySets As IEnumerable(Of String)) Implements ICache.InvalidateSets
        ' Go through the list of objects in each of the set. 
        For Each dep In entitySets
            For Each _Val In _cache.GetObjectsInRegion(dep)
                _cache.Remove(_Val.Key)
            Next _Val
        Next dep
    End Sub

    Public Sub InvalidateItem(ByVal key As String) Implements ICache.InvalidateItem
        key = GetCacheKey(key)

        Dim item As DataCacheItem = _cache.GetCacheItem(key)
        _cache.Remove(key)

        For Each tag In item.Tags
            _cache.Remove(key, tag.ToString())
        Next tag
    End Sub

    'creates a hash of the query to store as the key  
    Private Shared Function GetCacheKey(ByVal query As String) As String
        Dim bytes() As Byte = Encoding.UTF8.GetBytes(query)
        Dim hashString As String = Convert.ToBase64String(MD5.Create().ComputeHash(bytes))
        Return hashString
    End Function

    Private Sub CreateRegionIfNeeded(ByVal regionName As String)
        Try
            _cache.CreateRegion(regionName)
        Catch de As DataCacheException
            If de.ErrorCode <> DataCacheErrorCode.RegionAlreadyExists Then
                Throw
            End If
        End Try
    End Sub
End Class