﻿Imports System
Imports System.Linq
Imports EFCachingProvider
Imports EFCachingProvider.Caching
Imports EFAppFabricCacheAdapter
Imports System.Diagnostics
Imports EFTracingProvider
Imports Microsoft.ApplicationServer.Caching
Imports System.Security
Imports System.Collections.Generic
Imports NorthwindModelDbFirst

Module Module1

    Sub Main()
        'NOTE: Be sure that AppFabric is running. I use 
        'http://mdcadmintool.codeplex.com/ (run as admin!)

        ' Create and return the data cache
        Dim dataCache As ICache = CreateAppFabricCache()
        ' set up default cache and caching policy
        EFCachingProviderConfiguration.DefaultCache = dataCache
        EFCachingProviderConfiguration.DefaultCachingPolicy = CachingPolicy.CacheAll

        '// log SQL from all connections to the console
        'EFTracingProviderConfiguration.LogToConsole = true

        SimpleCachingDemo()
        CacheInvalidationDemo()
        'NonDeterministicQueryCachingDemo()

        Console.WriteLine("Demos Complete. Press any key to continue...")
        Console.ReadKey()
    End Sub

    ''' <summary>
    ''' In this demo we are running a set of queries 3 times and logging SQL commands to the console.
    ''' Note that queries are actually executed only in the first pass, while in second and third they are fulfilled
    ''' completely from the cache.
    ''' </summary>
    Private Sub SimpleCachingDemo()
        EFTracingProviderConfiguration.LogToConsole = True
        For i = 0 To 2
            Console.WriteLine()
            Console.WriteLine("*** Pass #{0}...", i)
            Console.WriteLine()
            Using context = New ExtendedNorthwindEntities()
                Console.WriteLine("Loading customer...")
                Dim cust = context.Customers.First(Function(c) c.CustomerID = "ALFKI")
                Console.WriteLine("Customer name: {0}", cust.ContactName)
                Console.WriteLine("Loading orders...")
                cust.Orders.Load()
                Console.WriteLine("Order count: {0}", cust.Orders.Count)
            End Using
        Next i

        Console.WriteLine("Finish reading from cache")


    End Sub

    ''' <summary>
    ''' In this demo we are running a set of queries and updates and 3 times and logging SQL commands to the console.
    ''' Notice how performing an update on Customer table causes the cache entry to be invalidated so we get 
    ''' a query in each pass. Because we aren't modifying OrderDetails table, the collection of order details
    ''' for the customer doesn't require a query in second and third pass.
    ''' </summary>
    Private Sub CacheInvalidationDemo()
        For i = 0 To 2
            Console.WriteLine()
            Console.WriteLine("*** Pass #{0}...", i)
            Console.WriteLine()
            Using context = New ExtendedNorthwindEntities()
                Console.WriteLine("Loading customer...")
                Dim cust = context.Customers.First(Function(c) c.CustomerID = "ANTON")
                Console.WriteLine("Customer name: {0}", cust.ContactName)
                cust.ContactName = "Change" & Environment.TickCount
                Console.WriteLine("Loading orders...")
                cust.Orders.Load()
                Console.WriteLine("Order count: {0}", cust.Orders.Count)
                context.SaveChanges()
            End Using
        Next i

        Console.WriteLine()
    End Sub

    ''' <summary>
    ''' In this demo we are a query based on time (whose results are non-deterministic, because it uses DateTime.Now)
    ''' Such queries are not cached, so we get a store query every time.
    '''
    ''' Note that non-cacheable queries don't count as cache misses.
    ''' </summary>
    Private Sub NonDeterministicQueryCachingDemo()
        For i = 0 To 2
            Console.WriteLine()
            Console.WriteLine("*** Pass #{0}...", i)
            Console.WriteLine()
            Using context = New ExtendedNorthwindEntities()
                Console.WriteLine("Loading orders...")
                context.Orders.Where(Function(c) c.ShippedDate < Date.Now).ToList()
            End Using
        Next i

        Console.WriteLine()
    End Sub

    'Private Shared Function CreateAppFabricCache(ByVal useLocalCache As Boolean) As ICache
    '       Dim endpoint As New DataCacheServerEndpoint("localhost", 22233, "DistributedCacheService")
    '       Dim fac As New DataCacheFactory(New DataCacheServerEndpoint() {endpoint}, useLocalCache, useLocalCache)

    '       Return New AppFabricCache(fac.GetCache("AppFabric"))
    '   End Function

    Private Function CreateAppFabricCache() As ICache
        ' Declare an array for the cache host
        Dim server = New List(Of DataCacheServerEndpoint)
        server.Add(New DataCacheServerEndpoint("localhost", 22233))
        ' Set up the DataCacheFactory configuration
        Dim conf = New DataCacheFactoryConfiguration
        conf.Servers = server
        Dim fac As New DataCacheFactory(conf)
        Return New AppFabricCache(fac.GetDefaultCache())

    End Function


    Private Function Secure(ByVal token As String) As SecureString
        Dim secureString As New SecureString()
        For Each c In token
            secureString.AppendChar(c)
        Next c
        secureString.MakeReadOnly()
        Return secureString
    End Function
End Module
