﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System

Namespace Caching
    ''' <summary>
    ''' Implementation of <see cref="CachingPolicy"/> which caches all queries of all size.
    ''' </summary>
    Friend Class CacheAllPolicy
        Inherits CachingPolicy
        ''' <summary>
        ''' Determines whether the specified command definition can be cached.
        ''' </summary>
        ''' <param name="commandDefinition">The command definition.</param>
        ''' <returns>
        ''' A value of <c>true</c> if the specified command definition can be cached; otherwise, <c>false</c>.
        ''' </returns>
        Protected Friend Overrides Function CanBeCached(ByVal commandDefinition As EFCachingCommandDefinition) As Boolean
            Return True
        End Function

        ''' <summary>
        ''' Gets the minimum and maximum number cacheable rows for a given command definition.
        ''' </summary>
        ''' <param name="cachingCommandDefinition">The command definition.</param>
        ''' <param name="minCacheableRows">The minimum number of cacheable rows.</param>
        ''' <param name="maxCacheableRows">The maximum number of cacheable rows.</param>
        Protected Friend Overrides Sub GetCacheableRows(ByVal cachingCommandDefinition As EFCachingCommandDefinition, <System.Runtime.InteropServices.Out()> ByRef minCacheableRows As Integer, <System.Runtime.InteropServices.Out()> ByRef maxCacheableRows As Integer)
            minCacheableRows = 0
            maxCacheableRows = Int32.MaxValue
        End Sub
    End Class
End Namespace