﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System

Namespace Caching
    ''' <summary>
    ''' Caching policy.
    ''' </summary>
    Public MustInherit Class CachingPolicy
        ''' <summary>
        ''' Initializes static members of the CachingPolicy class.
        ''' </summary>
        Shared Sub New()
            NoCaching = New NoCachingPolicy()
            CacheAll = New CacheAllPolicy()
        End Sub

        ''' <summary>
        ''' Gets the caching policy which does no caching.
        ''' </summary>
        ''' The no caching.
        Private Shared privateNoCaching As CachingPolicy
        Public Shared Property NoCaching() As CachingPolicy
            Get
                Return privateNoCaching
            End Get
            Private Set(ByVal value As CachingPolicy)
                privateNoCaching = value
            End Set
        End Property

        ''' <summary>
        ''' Gets the caching policy which caches results of all queries.
        ''' </summary>
        ''' The no caching.
        Private Shared privateCacheAll As CachingPolicy
        Public Shared Property CacheAll() As CachingPolicy
            Get
                Return privateCacheAll
            End Get
            Private Set(ByVal value As CachingPolicy)
                privateCacheAll = value
            End Set
        End Property

        ''' <summary>
        ''' Determines whether the specified command definition can be cached.
        ''' </summary>
        ''' <param name="commandDefinition">The command definition.</param>
        ''' <returns>
        ''' A value of <c>true</c> if the specified command definition can be cached; otherwise, <c>false</c>.
        ''' </returns>
        Protected Friend MustOverride Function CanBeCached(ByVal commandDefinition As EFCachingCommandDefinition) As Boolean

        ''' <summary>
        ''' Gets the minimum and maximum number cacheable rows for a given command definition.
        ''' </summary>
        ''' <param name="cachingCommandDefinition">The command definition.</param>
        ''' <param name="minCacheableRows">The minimum number of cacheable rows.</param>
        ''' <param name="maxCacheableRows">The maximum number of cacheable rows.</param>
        Protected Friend MustOverride Sub GetCacheableRows(ByVal cachingCommandDefinition As EFCachingCommandDefinition, <System.Runtime.InteropServices.Out()> ByRef minCacheableRows As Integer, <System.Runtime.InteropServices.Out()> ByRef maxCacheableRows As Integer)

        ''' <summary>
        ''' Gets the expiration timeout for a given command definition.
        ''' </summary>
        ''' <param name="cachingCommandDefinition">The command definition.</param>
        ''' <param name="slidingExpiration">The sliding expiration time.</param>
        ''' <param name="absoluteExpiration">The absolute expiration time.</param>
        Protected Friend Overridable Sub GetExpirationTimeout(ByVal cachingCommandDefinition As EFCachingCommandDefinition, <System.Runtime.InteropServices.Out()> ByRef slidingExpiration As TimeSpan, <System.Runtime.InteropServices.Out()> ByRef absoluteExpiration As Date)
            slidingExpiration = TimeSpan.Zero
            absoluteExpiration = Date.MaxValue
        End Sub
    End Class
End Namespace