﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text

Namespace Caching
    ''' <summary>
    ''' Interface to be implemented by cache implementations.
    ''' </summary>
    Public Interface ICache
        ''' <summary>
        ''' Tries to the get cached entry by key.
        ''' </summary>
        ''' <param name="key">The cache key.</param>
        ''' <param name="value">The retrieved value.</param>
        ''' <returns>A value of <c>true</c> if entry was found in the cache, <c>false</c> otherwise.</returns>
        Function GetItem(ByVal key As String, <System.Runtime.InteropServices.Out()> ByRef value As Object) As Boolean

        ''' <summary>
        ''' Adds the specified entry to the cache.
        ''' </summary>
        ''' <param name="key">The entry key.</param>
        ''' <param name="value">The entry value.</param>
        ''' <param name="dependentEntitySets">The list of dependent entity sets.</param>
        ''' <param name="slidingExpiration">The sliding expiration.</param>
        ''' <param name="absoluteExpiration">The absolute expiration.</param>
        Sub PutItem(ByVal key As String, ByVal value As Object, ByVal dependentEntitySets As IEnumerable(Of String), ByVal slidingExpiration As TimeSpan, ByVal absoluteExpiration As Date)

        ''' <summary>
        ''' Invalidates all cache entries which are dependent on any of the specified entity sets.
        ''' </summary>
        ''' <param name="entitySets">The entity sets.</param>
        Sub InvalidateSets(ByVal entitySets As IEnumerable(Of String))

        ''' <summary>
        ''' Invalidates cache entry with a given key.
        ''' </summary>
        ''' <param name="key">The cache key.</param>
        Sub InvalidateItem(ByVal key As String)
    End Interface
End Namespace