﻿
' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Data.Common.CommandTrees


''' <summary>
''' General purpose command tree visitor that does not return any value.
''' </summary>
Friend MustInherit Class DbCommandTreeScanner
    Inherits DbExpressionVisitor
    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbVariableReferenceExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbVariableReferenceExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbVariableReferenceExpression)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbUnionAllExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbUnionAllExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbUnionAllExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Left.Accept(Me)
        expression.Right.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbTreatExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbTreatExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbTreatExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbSortExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbSortExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbSortExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Input.Expression.Accept(Me)
        For Each sortClause In expression.SortOrder
            sortClause.Expression.Accept(Me)
        Next sortClause
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbSkipExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbSkipExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbSkipExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Count.Accept(Me)
        expression.Input.Expression.Accept(Me)
        For Each sortClause In expression.SortOrder
            sortClause.Expression.Accept(Me)
        Next sortClause
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbScanExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbScanExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbScanExpression)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbRelationshipNavigationExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbRelationshipNavigationExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbRelationshipNavigationExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.NavigationSource.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbRefExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbRefExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbRefExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbQuantifierExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbQuantifierExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbQuantifierExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Input.Expression.Accept(Me)
        expression.Predicate.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbPropertyExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbPropertyExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbPropertyExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Instance.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbProjectExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbProjectExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbProjectExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Projection.Accept(Me)
        expression.Input.Expression.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbParameterReferenceExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbParameterReferenceExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbParameterReferenceExpression)
        ' expression.
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbOrExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbOrExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbOrExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Left.Accept(Me)
        expression.Right.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbOfTypeExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbOfTypeExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbOfTypeExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbNullExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbNullExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbNullExpression)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbNotExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbNotExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbNotExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbNewInstanceExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbNewInstanceExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbNewInstanceExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        For Each e In expression.Arguments
            e.Accept(Me)
        Next e
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbLimitExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbLimitExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbLimitExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
        expression.Limit.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbLikeExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbLikeExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbLikeExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
        expression.Pattern.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbJoinExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbJoinExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbJoinExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Left.Expression.Accept(Me)
        expression.Right.Expression.Accept(Me)
        expression.JoinCondition.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbIsOfExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbIsOfExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbIsOfExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbIsNullExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbIsNullExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbIsNullExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbIsEmptyExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbIsEmptyExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbIsEmptyExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbIntersectExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbIntersectExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbIntersectExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Left.Accept(Me)
        expression.Right.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbGroupByExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbGroupByExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbGroupByExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Input.Expression.Accept(Me)
        For Each ex In expression.Keys
            ex.Accept(Me)
        Next ex

        For Each agg In expression.Aggregates
            For Each ex In agg.Arguments
                ex.Accept(Me)
            Next ex
        Next agg
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbRefKeyExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbRefKeyExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbRefKeyExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbEntityRefExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbEntityRefExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbEntityRefExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbFunctionExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbFunctionExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbFunctionExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        For Each ex In expression.Arguments
            ex.Accept(Me)
        Next ex
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbFilterExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbFilterExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbFilterExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Input.Expression.Accept(Me)
        expression.Predicate.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbExceptExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbExceptExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbExceptExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Left.Accept(Me)
        expression.Right.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbElementExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbElementExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbElementExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbDistinctExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbDistinctExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbDistinctExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbDerefExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbDerefExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbDerefExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbCrossJoinExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbCrossJoinExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbCrossJoinExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        For Each binding In expression.Inputs
            binding.Expression.Accept(Me)
        Next binding
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbConstantExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbConstantExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbConstantExpression)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbComparisonExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.Db ComparisonExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbComparisonExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Left.Accept(Me)
        expression.Right.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbCastExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbCastExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbCastExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Argument.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbCaseExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbCaseExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbCaseExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        For Each e In expression.When
            e.Accept(Me)
        Next e

        For Each e In expression.Then
            e.Accept(Me)
        Next e

        expression.Else.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbArithmeticExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbArithmeticExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbArithmeticExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        For Each ex In expression.Arguments
            ex.Accept(Me)
        Next ex
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbApplyExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbApplyExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbApplyExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Apply.Expression.Accept(Me)
        expression.Input.Expression.Accept(Me)
    End Sub

    ''' <summary>
    ''' Implements the visitor pattern for <see cref="T:System.Data.Common.CommandTrees.DbAndExpression"/>.
    ''' </summary>
    ''' <param name="expression">The <see cref="T:System.Data.Common.CommandTrees.DbAndExpression"/> that is visited.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbAndExpression)
        If expression Is Nothing Then
            Throw New ArgumentNullException("expression")
        End If

        expression.Left.Accept(Me)
        expression.Right.Accept(Me)
    End Sub

    ''' <summary>
    ''' Handles any expression of an unrecognized type.
    ''' </summary>
    ''' <param name="expression">The expression to be handled.</param>
    Public Overloads Overrides Sub Visit(ByVal expression As DbExpression)
        Throw New NotSupportedException()
    End Sub
End Class