﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System.Data
Imports System.Data.Common
Imports EFCachingProvider.Caching
Imports EFProviderWrapperToolkit

''' <summary>
''' Implementation of <see cref="DbConnection"/> with support for caching of Entity Framework queries.
''' </summary>
Public Class EFCachingConnection
    Inherits DbConnectionWrapper
    ''' <summary>
    ''' Initializes a new instance of the EFCachingConnection class.
    ''' </summary>
    Public Sub New()
        Me.New(Nothing)
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the EFCachingConnection class.
    ''' </summary>
    ''' <param name="wrappedConnection">The wrapped connection.</param>
    Public Sub New(ByVal wrappedConnection As DbConnection)
        Me.WrappedConnection = wrappedConnection
        Me.CachingPolicy = EFCachingProviderConfiguration.DefaultCachingPolicy
        Me.Cache = EFCachingProviderConfiguration.DefaultCache
    End Sub

    ''' <summary>
    ''' Gets or sets the cache.
    ''' </summary>
    ''' <value>The cache.</value>
    Public Property Cache() As ICache

    ''' <summary>
    ''' Gets or sets the caching policy.
    ''' </summary>
    ''' <value>The caching policy.</value>
    Public Property CachingPolicy() As CachingPolicy

    ''' <summary>
    ''' Gets the name of the default wrapped provider.
    ''' </summary>
    ''' <returns>Name of the default wrapped provider.</returns>
    Protected Overrides ReadOnly Property DefaultWrappedProviderName() As String
        Get
            Return EFCachingProviderConfiguration.DefaultWrappedProvider
        End Get
    End Property

    ''' <summary>
    ''' Gets the <see cref="T:System.Data.Common.DbProviderFactory"/> for this <see cref="T:System.Data.Common.DbConnection"/>.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' A <see cref="T:System.Data.Common.DbProviderFactory"/>.
    ''' </returns>
    Protected Overrides ReadOnly Property DbProviderFactory() As DbProviderFactory
        Get
            Return EFCachingProviderFactory.Instance
        End Get
    End Property

    ''' <summary>
    ''' Starts a database transaction.
    ''' </summary>
    ''' <param name="isolationLevel">Specifies the isolation level for the transaction.</param>
    ''' <returns>
    ''' An object representing the new transaction.
    ''' </returns>
    Protected Overrides Function BeginDbTransaction(ByVal isolationLevel As IsolationLevel) As DbTransaction
        Return New EFCachingTransaction(WrappedConnection.BeginTransaction(isolationLevel), Me)
    End Function
End Class