﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Data
Imports EFCachingProvider.Caching

''' <summary>
''' Implementation of which returns results from <see cref="DbQueryResults"/> object.
''' </summary>
Friend Class CachingDataReaderCacheReader
    Inherits EFCachingDataReaderBase
    Private queryResults As DbQueryResults
    Private currentRow As Integer
    Private _isClosed As Boolean

    ''' <summary>
    ''' Initializes a new instance of the CachingDataReaderCacheReader class.
    ''' </summary>
    ''' <param name="queryResults">The query results.</param>
    Public Sub New(ByVal queryResults As DbQueryResults)
        Me.queryResults = queryResults
    End Sub

    ''' <summary>
    ''' Gets a value that indicates whether this <see cref="T:System.Data.Common.DbDataReader"/> contains one or more rows.
    ''' </summary>
    ''' <value></value>
    ''' <returns>true if the <see cref="T:System.Data.Common.DbDataReader"/> contains one or more rows; otherwise false.
    ''' </returns>
    Public Overrides ReadOnly Property HasRows() As Boolean
        Get
            Return Me.queryResults.Rows.Count > 0
        End Get
    End Property

    ''' <summary>
    ''' Gets a value indicating whether the <see cref="T:System.Data.Common.DbDataReader"/> is closed.
    ''' </summary>
    ''' <value></value>
    ''' <returns>true if the <see cref="T:System.Data.Common.DbDataReader"/> is closed; otherwise false.
    ''' </returns>
    Public Overrides ReadOnly Property IsClosed() As Boolean
        Get
            Return Me._isClosed
        End Get
    End Property

    ''' <summary>
    ''' Gets a value indicating the depth of nesting for the current row.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The depth of nesting for the current row.
    ''' </returns>
    Public Overrides ReadOnly Property Depth() As Integer
        Get
            Throw New NotImplementedException()
        End Get
    End Property

    ''' <summary>
    ''' Gets the number of rows changed, inserted, or deleted by execution of the SQL statement.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The number of rows changed, inserted, or deleted. -1 for SELECT statements; 0 if no rows were affected or the statement failed.
    ''' </returns>
    Public Overrides ReadOnly Property RecordsAffected() As Integer
        Get
            Throw New NotImplementedException()
        End Get
    End Property

    ''' <summary>
    ''' Gets name of the data type of the specified column.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>
    ''' A string representing the name of the data type.
    ''' </returns>
    Public Overrides Function GetDataTypeName(ByVal ordinal As Integer) As String
        Throw New NotImplementedException()
    End Function

    ''' <summary>
    ''' Gets the data type of the specified column.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The data type of the specified column.</returns>
    Public Overrides Function GetFieldType(ByVal ordinal As Integer) As Type
        Throw New NotImplementedException()
    End Function

    ''' <summary>
    ''' Gets the name of the column, given the zero-based column ordinal.
    ''' </summary>
    ''' <param name="ordinal">The zero-based column ordinal.</param>
    ''' <returns>The name of the specified column.</returns>
    Public Overrides Function GetName(ByVal ordinal As Integer) As String
        Return Me.queryResults.ColumnNames(ordinal)
    End Function

    ''' <summary>
    ''' Gets the column ordinal given the name of the column.
    ''' </summary>
    ''' <param name="name">The name of the column.</param>
    ''' <returns>The zero-based column ordinal.</returns>
    ''' <exception cref="T:System.IndexOutOfRangeException">
    ''' The name specified is not a valid column name.
    ''' </exception>
    Public Overrides Function GetOrdinal(ByVal name As String) As Integer
        Throw New NotImplementedException()
    End Function

    ''' <summary>
    ''' Returns a <see cref="T:System.Data.DataTable"/> that describes the column metadata of the <see cref="T:System.Data.Common.DbDataReader"/>.
    ''' </summary>
    ''' <returns>
    ''' A <see cref="T:System.Data.DataTable"/> that describes the column metadata.
    ''' </returns>
    Public Overrides Function GetSchemaTable() As DataTable
        Throw New NotImplementedException()
    End Function

    ''' <summary>
    ''' Advances the reader to the next result when reading the results of a batch of statements.
    ''' </summary>
    ''' <returns>
    ''' true if there are more result sets; otherwise false.
    ''' </returns>
    Public Overrides Function NextResult() As Boolean
        Return False
    End Function

    ''' <summary>
    ''' Closes the <see cref="T:System.Data.Common.DbDataReader"/> object.
    ''' </summary>
    Public Overrides Sub Close()
        Me._isClosed = True
    End Sub

    ''' <summary>
    ''' Advances the reader to the next record in a result set.
    ''' </summary>
    ''' <returns>
    ''' true if there are more rows; otherwise false.
    ''' </returns>
    Public Overrides Function Read() As Boolean
        If Me.currentRow < Me.queryResults.Rows.Count Then
            Dim values() As Object = Me.queryResults.Rows(Me.currentRow)
            Me.currentRow += 1
            SetValues(values)
            Return True
        Else
            Return False
        End If
    End Function
End Class