﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Data.Common
Imports System.Data.Objects
Imports EFCachingProvider.Caching
Imports EFProviderWrapperToolkit

''' <summary>
''' Extension methods for EFCachingProvider.
''' </summary>
Public Module EFCachingExtensionMethods
    ''' <summary>
    ''' Gets the cache associated with this connection.
    ''' </summary>
    ''' <param name="connection">The connection.</param>
    ''' <returns><see cref="ICache"/> implementation.</returns>
    <System.Runtime.CompilerServices.Extension()> _
    Public Function GetCache(ByVal connection As DbConnection) As ICache
        Return connection.UnwrapConnection(Of EFCachingConnection)().Cache
    End Function

    ''' <summary>
    ''' Gets the cache associated with the object context.
    ''' </summary>
    ''' <param name="context">The context.</param>
    ''' <returns>Object context.</returns>
    <System.Runtime.CompilerServices.Extension()> _
    Public Function GetCache(ByVal context As ObjectContext) As ICache
        If context Is Nothing Then
            Throw New ArgumentNullException("context")
        End If

        Return GetCache(context.Connection)
    End Function

    ''' <summary>
    ''' Sets the <see cref="ICache"/> implementation for given connection.
    ''' </summary>
    ''' <param name="connection">The connection.</param>
    ''' <param name="cache">The cache.</param>
    <System.Runtime.CompilerServices.Extension()> _
    Public Sub SetCache(ByVal connection As DbConnection, ByVal cache As ICache)
        connection.UnwrapConnection(Of EFCachingConnection)().Cache = cache
    End Sub

    ''' <summary>
    ''' Sets the <see cref="ICache"/> implementation for given object context.
    ''' </summary>
    ''' <param name="context">The object context.</param>
    ''' <param name="cache">The cache.</param>
    <System.Runtime.CompilerServices.Extension()> _
    Public Sub SetCache(ByVal context As ObjectContext, ByVal cache As ICache)
        If context Is Nothing Then
            Throw New ArgumentNullException("context")
        End If

        SetCache(context.Connection, cache)
    End Sub
End Module