﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System.Configuration
Imports EFCachingProvider.Caching
Imports EFProviderWrapperToolkit

''' <summary>
''' Default configuration settings for EFCachingProvider.
''' </summary>
Public NotInheritable Class EFCachingProviderConfiguration
    ''' <summary>
    ''' Initializes static members of the EFCachingProviderConfiguration class.
    ''' </summary>
    Private Sub New()
    End Sub
    Shared Sub New()
        DefaultCachingPolicy = CachingPolicy.NoCaching
        DefaultWrappedProvider = ConfigurationManager.AppSettings("EFCachingProvider.wrappedProvider")
    End Sub

    ''' <summary>
    ''' Gets or sets the default wrapped provider.
    ''' </summary>
    ''' <value>The default wrapped provider.</value>
    Public Shared Property DefaultWrappedProvider() As String

    ''' <summary>
    ''' Gets or sets default caching <see cref="ICache"/> implementation which should be used for new connections.
    ''' </summary>
    Public Shared Property DefaultCache() As ICache

    ''' <summary>
    ''' Gets or sets default caching policy to be applied to all new connections.
    ''' </summary>
    Public Shared Property DefaultCachingPolicy() As CachingPolicy

    ''' <summary>
    ''' Registers the provider factory 
    ''' </summary>
    Public Shared Sub RegisterProvider()
        DbProviderFactoryBase.RegisterProvider("EF Caching Data Provider", "EFCachingProvider", GetType(EFCachingProviderFactory))
    End Sub
End Class