﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Data.Common

''' <summary>
''' Provider factory for EFCachingProvider
''' </summary>
Public Class EFCachingProviderFactory
    Inherits DbProviderFactory
    Implements IServiceProvider
    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Security", "CA2104:DoNotDeclareReadOnlyMutableReferenceTypes", Justification:="Factory class is immutable.")>
    Public Shared ReadOnly Instance As New EFCachingProviderFactory()

    ''' <summary>
    ''' Specifies whether the specific <see cref="T:System.Data.Common.DbProviderFactory"/> supports the <see cref="T:System.Data.Common.DbDataSourceEnumerator"/> class.
    ''' </summary>
    ''' <value></value>
    ''' <returns>true if the instance of the <see cref="T:System.Data.Common.DbProviderFactory"/> supports the <see cref="T:System.Data.Common.DbDataSourceEnumerator"/> class; otherwise false.
    ''' </returns>
    Public Overrides ReadOnly Property CanCreateDataSourceEnumerator() As Boolean
        Get
            Return False
        End Get
    End Property

    ''' <summary>
    ''' Returns a new instance of the provider's class that implements the <see cref="T:System.Data.Common.DbCommand"/> class.
    ''' </summary>
    ''' <returns>
    ''' A new instance of <see cref="T:System.Data.Common.DbCommand"/>.
    ''' </returns>
    Public Overrides Function CreateCommand() As DbCommand
        Return Nothing
    End Function

    ''' <summary>
    ''' Returns a new instance of the provider's class that implements the <see cref="T:System.Data.Common.DbCommandBuilder"/> class.
    ''' </summary>
    ''' <returns>
    ''' A new instance of <see cref="T:System.Data.Common.DbCommandBuilder"/>.
    ''' </returns>
    Public Overrides Function CreateCommandBuilder() As DbCommandBuilder
        Throw New NotSupportedException()
    End Function

    ''' <summary>
    ''' Returns a new instance of the provider's class that implements the <see cref="T:System.Data.Common.DbConnection"/> class.
    ''' </summary>
    ''' <returns>
    ''' A new instance of <see cref="T:System.Data.Common.DbConnection"/>.
    ''' </returns>
    Public Overrides Function CreateConnection() As DbConnection
        Return New EFCachingConnection()
    End Function

    ''' <summary>
    ''' Returns a new instance of the provider's class that implements the <see cref="T:System.Data.Common.DbConnectionStringBuilder"/> class.
    ''' </summary>
    ''' <returns>
    ''' A new instance of <see cref="T:System.Data.Common.DbConnectionStringBuilder"/>.
    ''' </returns>
    Public Overrides Function CreateConnectionStringBuilder() As DbConnectionStringBuilder
        Throw New NotSupportedException()
    End Function

    ''' <summary>
    ''' Returns a new instance of the provider's class that implements the <see cref="T:System.Data.Common.DbDataAdapter"/> class.
    ''' </summary>
    ''' <returns>
    ''' A new instance of <see cref="T:System.Data.Common.DbDataAdapter"/>.
    ''' </returns>
    Public Overrides Function CreateDataAdapter() As DbDataAdapter
        Throw New NotSupportedException()
    End Function

    ''' <summary>
    ''' Returns a new instance of the provider's class that implements the <see cref="T:System.Data.Common.DbDataSourceEnumerator"/> class.
    ''' </summary>
    ''' <returns>
    ''' A new instance of <see cref="T:System.Data.Common.DbDataSourceEnumerator"/>.
    ''' </returns>
    Public Overrides Function CreateDataSourceEnumerator() As DbDataSourceEnumerator
        Return System.Data.Sql.SqlDataSourceEnumerator.Instance
    End Function

    ''' <summary>
    ''' Returns a new instance of the provider's class that implements the <see cref="T:System.Data.Common.DbParameter"/> class.
    ''' </summary>
    ''' <returns>
    ''' A new instance of <see cref="T:System.Data.Common.DbParameter"/>.
    ''' </returns>
    Public Overrides Function CreateParameter() As DbParameter
        Throw New NotSupportedException()
    End Function

    ''' <summary>
    ''' Gets the service object of the specified type.
    ''' </summary>
    ''' <param name="serviceType">An object that specifies the type of service object to get.</param>
    ''' <returns>
    ''' A service object of type <paramref name="serviceType"/>.
    ''' -or-
    ''' null if there is no service object of type <paramref name="serviceType"/>.
    ''' </returns>
    Public Function GetService(ByVal serviceType As Type) As Object Implements IServiceProvider.GetService
        If serviceType Is GetType(DbProviderServices) Then
            Return EFCachingProviderServices.Instance
        Else
            Return Nothing
        End If
    End Function
End Class