﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System.Data.Common
Imports System.Data.Common.CommandTrees
Imports EFProviderWrapperToolkit

''' <summary>
''' Implementation of <see cref="DbProviderServices"/> for EFCachingProvider.
''' </summary>
Friend Class EFCachingProviderServices
    Inherits DbProviderServicesBase
    ''' <summary>
    ''' Initializes static members of the EFCachingProviderServices class.
    ''' </summary>
    Shared Sub New()
        Instance = New EFCachingProviderServices()
    End Sub

    ''' <summary>
    ''' Prevents a default instance of the EFCachingProviderServices class from being created.
    ''' </summary>
    Private Sub New()
    End Sub

    ''' <summary>
    ''' Gets the singleton instance of <see cref="EFCachingProviderServices"/>.
    ''' </summary>
    '''The singleton instance.
    Private Shared privateInstance As EFCachingProviderServices
    Public Shared Property Instance() As EFCachingProviderServices
        Get
            Return privateInstance
        End Get
        Private Set(ByVal value As EFCachingProviderServices)
            privateInstance = value
        End Set
    End Property

    ''' <summary>
    ''' Gets the default name of the wrapped provider.
    ''' </summary>
    ''' <returns>
    ''' Default name of the wrapped provider (to be used when
    ''' provider is not specified in the connction string)
    ''' </returns>
    Protected Overrides ReadOnly Property DefaultWrappedProviderName() As String
        Get
            Return EFCachingProviderConfiguration.DefaultWrappedProvider
        End Get
    End Property

    ''' <summary>
    ''' Gets the provider invariant iname.
    ''' </summary>
    ''' <returns>Provider invariant name.</returns>
    Protected Overrides ReadOnly Property ProviderInvariantName() As String
        Get
            Return "EFCachingProvider"
        End Get
    End Property

    ''' <summary>
    ''' Creates the command definition wrapper.
    ''' </summary>
    ''' <param name="wrappedCommandDefinition">The wrapped command definition.</param>
    ''' <param name="commandTree">The command tree.</param>
    ''' <returns>
    ''' The <see cref="DbCommandDefinitionWrapper"/> object.
    ''' </returns>
    Public Overrides Function CreateCommandDefinitionWrapper(ByVal wrappedCommandDefinition As DbCommandDefinition, ByVal commandTree As DbCommandTree) As DbCommandDefinitionWrapper
        Return New EFCachingCommandDefinition(wrappedCommandDefinition, commandTree)
    End Function
End Class