﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Data.Common
Imports System.Data.Common.CommandTrees

''' <summary>
''' Implementation of common methods of <see cref="DbCommandDefinition"/> class.
''' </summary>
Public Class DbCommandDefinitionWrapper
    Inherits DbCommandDefinition

    Private _wrappedCommandDefinition As DbCommandDefinition

    Private _commandTree As DbCommandTree
    Private commandCreator As Func(Of DbCommand, DbCommandDefinitionWrapper, DbCommand)

    ''' <summary>
    ''' Initializes a new instance of the DbCommandDefinitionWrapper class.
    ''' </summary>
    ''' <param name="wrappedCommandDefinition">The wrapped command definition.</param>
    ''' <param name="commandTree">The command tree.</param>
    ''' <param name="commandCreator">The command creator delegate.</param>
    Public Sub New(ByVal wrappedCommandDefinition As DbCommandDefinition, ByVal commandTree As DbCommandTree, ByVal commandCreator As Func(Of DbCommand, DbCommandDefinitionWrapper, DbCommand))
        Me._wrappedCommandDefinition = wrappedCommandDefinition
        Me._commandTree = commandTree
        Me.commandCreator = commandCreator
    End Sub

    ''' <summary>
    ''' Gets the command tree.
    ''' </summary>
    ''' <value>The command tree.</value>
    Public ReadOnly Property CommandTree() As DbCommandTree
        Get
            Return Me._commandTree
        End Get
    End Property

    ''' <summary>
    ''' Gets the wrapped command definition.
    ''' </summary>
    ''' <value>The wrapped command definition.</value>
    Protected ReadOnly Property WrappedCommandDefinition() As DbCommandDefinition
        Get
            Return Me._wrappedCommandDefinition
        End Get
    End Property

    ''' <summary>
    ''' Creates and returns a <see cref="T:System.Data.Common.DbCommandDefinition"/> object associated with the current connection.
    ''' </summary>
    ''' <returns>
    ''' A <see cref="T:System.Data.Common.DbCommandDefinition"/>.
    ''' </returns>
    Public Overrides Function CreateCommand() As DbCommand
        Return Me.commandCreator(Me._wrappedCommandDefinition.CreateCommand(), Me)
    End Function
End Class