﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System.Data.Common
Imports System.Xml
Imports System.Xml.Linq

''' <summary>
''' Wrapper for <see cref="DbProviderManifest"/> objects.
''' </summary>
Public Class DbProviderManifestWrapper
    Inherits DbProviderManifest
    Private providerInvariantName As String

    Private _wrappedProviderManifest As DbProviderManifest
    Private wrapperProviderInvariantName As String

    ''' <summary>
    ''' Initializes a new instance of the DbProviderManifestWrapper class.
    ''' </summary>
    ''' <param name="wrapperProviderInvariantName">Wrapper provider invariant name.</param>
    ''' <param name="providerInvariantName">Provider invariant name.</param>
    ''' <param name="wrappedProviderManifest">The wrapped provider manifest.</param>
    Public Sub New(ByVal wrapperProviderInvariantName As String, ByVal providerInvariantName As String, ByVal wrappedProviderManifest As DbProviderManifest)
        Me.wrapperProviderInvariantName = wrapperProviderInvariantName
        Me.providerInvariantName = providerInvariantName
        Me._wrappedProviderManifest = wrappedProviderManifest
    End Sub

    ''' <summary>
    ''' Gets the namespace name supported by this provider manifest.
    ''' </summary>
    ''' <value></value>
    ''' <returns>The namespace name supported by this provider manifest.</returns>
    Public Overrides ReadOnly Property NamespaceName() As String
        Get
            Return Me._wrappedProviderManifest.NamespaceName
        End Get
    End Property

    ''' <summary>
    ''' Gets the wrapped provider manifest.
    ''' </summary>
    ''' <value>The wrapped provider manifest.</value>
    Public ReadOnly Property WrappedProviderManifest() As DbProviderManifest
        Get
            Return Me._wrappedProviderManifest
        End Get
    End Property

    ''' <summary>
    ''' Gets the invariant name of the wrapped provider.
    ''' </summary>
    ''' <value>The invariant name of the wrapped provider.</value>
    Public ReadOnly Property WrappedProviderManifestInvariantName() As String
        Get
            Return Me.providerInvariantName
        End Get
    End Property

    ''' <summary>
    ''' Returns the best mapped equivalent EDM type for a specified storage type.
    ''' </summary>
    ''' <param name="storeType">The instance of the <see cref="T:System.Data.Metadata.Edm.TypeUsage"/> class that encapsulates both a storage type and a set of facets for that type.</param>
    ''' <returns>
    ''' The instance of that <see cref="T:System.Data.Metadata.Edm.TypeUsage"/> encapsulates both an EDM type and a set of facets for that type.
    ''' </returns>
    Public Overrides Function GetEdmType(ByVal storeType As System.Data.Metadata.Edm.TypeUsage) As System.Data.Metadata.Edm.TypeUsage
        Return Me._wrappedProviderManifest.GetEdmType(storeType)
    End Function

    ''' <summary>
    ''' Returns the list of facet descriptions for the specified Entity Data Model (EDM) type.
    ''' </summary>
    ''' <param name="edmType">An <see cref="T:System:Data.Metadata.Edm.EdmType"/> for which the facet descriptions are to be retrieved.</param>
    ''' <returns>
    ''' A collection of type <see cref="T:System.Collections.ObjectModel.ReadOnlyCollection`1"/> that contains the list of facet descriptions for the specified Entity Data Model (EDM) type.
    ''' </returns>
    Public Overrides Function GetFacetDescriptions(ByVal edmType As System.Data.Metadata.Edm.EdmType) As System.Collections.ObjectModel.ReadOnlyCollection(Of System.Data.Metadata.Edm.FacetDescription)
        Return Me._wrappedProviderManifest.GetFacetDescriptions(edmType)
    End Function

    ''' <summary>
    ''' Returns the list of provider-specific functions.
    ''' </summary>
    ''' <returns>
    ''' A collection of type <see cref="T:System.Collections.ObjectModel.ReadOnlyCollection`1"/> that contains the list of provider-specific functions.
    ''' </returns>
    Public Overrides Function GetStoreFunctions() As System.Collections.ObjectModel.ReadOnlyCollection(Of System.Data.Metadata.Edm.EdmFunction)
        Return Me._wrappedProviderManifest.GetStoreFunctions()
    End Function

    ''' <summary>
    ''' Returns the best mapped equivalent storage type for a specified Entity Data Model (EDM) type.
    ''' </summary>
    ''' <param name="edmType">The instance of the <see cref="T:System.Data.Metadata.Edm.TypeUsage"/> class that encapsulates both an EDM type and a set of facets for that type.</param>
    ''' <returns>
    ''' The instance of the <see cref="T:System.Data.Metadata.Edm.TypeUsage"/> class that encapsulates both a storage type and a set of facets for that type.
    ''' </returns>
    Public Overrides Function GetStoreType(ByVal edmType As System.Data.Metadata.Edm.TypeUsage) As System.Data.Metadata.Edm.TypeUsage
        Return Me._wrappedProviderManifest.GetStoreType(edmType)
    End Function

    ''' <summary>
    ''' Returns the list of primitive types that are supported by the storage provider.
    ''' </summary>
    ''' <returns>
    ''' A collection of type <see cref="T:System.Collections.ObjectModel.ReadOnlyCollection`1"/> that contains the list of primitive types supported by the storage provider.
    ''' </returns>
    Public Overrides Function GetStoreTypes() As System.Collections.ObjectModel.ReadOnlyCollection(Of System.Data.Metadata.Edm.PrimitiveType)
        Return Me._wrappedProviderManifest.GetStoreTypes()
    End Function

    ''' <summary>
    ''' Returns a <see cref="System.String"/> that represents this instance.
    ''' </summary>
    ''' <returns>
    ''' A <see cref="System.String"/> that represents this instance.
    ''' </returns>
    Public Overrides Function ToString() As String
        Return "DbProviderManifestWrapper(Wrapped=" & Me._wrappedProviderManifest.ToString() & ")"
    End Function

    ''' <summary>
    ''' Indicates if the provider supports escaping strings to be used as patterns in a Like expression.
    ''' </summary>
    ''' <param name="escapeCharacter">If the provider supports escaping, the character that would be used as the escape character.</param>
    ''' <returns>
    ''' Returns true if this provider supports escaping strings to be used as patterns in a Like expression; otherwise, false. The default implementation returns false.
    ''' </returns>
    Public Overrides Function SupportsEscapingLikeArgument(<System.Runtime.InteropServices.Out()> ByRef escapeCharacter As Char) As Boolean
        Return Me._wrappedProviderManifest.SupportsEscapingLikeArgument(escapeCharacter)
    End Function

    ''' <summary>
    ''' Provider writers should override this method to return the argument with the wildcards and the escape  character escaped. This method is only used if <see cref="M:System.Data.Common.DbProviderManifest.SupportsEscapingLikeArgument(System.Char@)"/> returns true.
    ''' </summary>
    ''' <param name="argument">The argument to be escaped.</param>
    ''' <returns>
    ''' The argument with the wildcards and the escape character escaped.
    ''' </returns>
    Public Overrides Function EscapeLikeArgument(ByVal argument As String) As String
        Return Me._wrappedProviderManifest.EscapeLikeArgument(argument)
    End Function

    ''' <summary>
    ''' Returns an XML reader that represents the storage provider-specific information.
    ''' </summary>
    ''' <param name="informationType">The name of the information to be retrieved. Providers are required to support the following values: <see cref="F:System.Data.Common.DbProviderManifest.StoreSchemaDefinition"/> or <see cref="F:System.Data.Common.DbProviderManifest.StoreSchemaMapping"/>.</param>
    ''' <returns>
    ''' An <see cref="T:System.Xml.XMLReader"/> object that represents the storage provider-specific information.
    ''' </returns>
    Protected Overrides Function GetDbInformation(ByVal informationType As String) As System.Xml.XmlReader
        If informationType = DbProviderManifest.StoreSchemaDefinition Then
            Return Me.InjectProviderNameIntoSsdl(Me._wrappedProviderManifest.GetInformation(informationType))
        End If

        Return Me._wrappedProviderManifest.GetInformation(informationType)
    End Function

    ''' <summary>
    ''' Injects the provider name into SSDL.
    ''' </summary>
    ''' <param name="originalReader">The original reader for the XML stream representing SSDL.</param>
    ''' <returns>XmlReader for the modified SSDL</returns>
    Private Function InjectProviderNameIntoSsdl(ByVal originalReader As XmlReader) As XmlReader
        Dim ssdl As XElement = XElement.Load(originalReader)
        ssdl.Attribute("ProviderManifestToken").Value = ssdl.Attribute("Provider").Value & ";" & ssdl.Attribute("ProviderManifestToken").Value
        ssdl.Attribute("Provider").Value = Me.wrapperProviderInvariantName
        Return ssdl.CreateReader()
    End Function
End Class