﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System.Data.Common

''' <summary>
''' Implementation of common methods for <see cref="DbTransaction"/> wrappers.
''' </summary>
Public MustInherit Class DbTransactionWrapper
    Inherits DbTransaction
    Private _wrappedTransaction As DbTransaction
    Private _connectionWrapper As DbConnectionWrapper

    ''' <summary>
    ''' Initializes a new instance of the <see cref="DbTransactionWrapper"/> class.
    ''' </summary>
    ''' <param name="wrappedTransaction">The wrapped transaction.</param>
    ''' <param name="connection">The connection.</param>
    Protected Sub New(ByVal wrappedTransaction As DbTransaction, ByVal connection As DbConnectionWrapper)
        Me._wrappedTransaction = wrappedTransaction
        Me._connectionWrapper = connection
    End Sub

    ''' <summary>
    ''' Specifies the <see cref="T:System.Data.IsolationLevel"/> for this transaction.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The <see cref="T:System.Data.IsolationLevel"/> for this transaction.
    ''' </returns>
    Public Overrides ReadOnly Property IsolationLevel() As System.Data.IsolationLevel
        Get
            Return Me._wrappedTransaction.IsolationLevel
        End Get
    End Property

    ''' <summary>
    ''' Gets the wrapped transaction.
    ''' </summary>
    ''' <value>The wrapped transaction.</value>
    Public ReadOnly Property WrappedTransaction() As DbTransaction
        Get
            Return Me._wrappedTransaction
        End Get
    End Property

    ''' <summary>
    ''' Specifies the <see cref="T:System.Data.Common.DbConnection"/> object associated with the transaction.
    ''' </summary>
    ''' <value></value>
    ''' <returns>
    ''' The <see cref="T:System.Data.Common.DbConnection"/> object associated with the transaction.
    ''' </returns>
    Protected Overrides ReadOnly Property DbConnection() As DbConnection
        Get
            Return Me._connectionWrapper
        End Get
    End Property

    ''' <summary>
    ''' Gets the connection wrapper.
    ''' </summary>
    ''' <value>The connection wrapper.</value>
    Protected ReadOnly Property ConnectionWrapper() As DbConnectionWrapper
        Get
            Return Me._connectionWrapper
        End Get
    End Property

    ''' <summary>
    ''' Commits the database transaction.
    ''' </summary>
    Public Overrides Sub Commit()
        Me._wrappedTransaction.Commit()
    End Sub

    ''' <summary>
    ''' Rolls back a transaction from a pending state.
    ''' </summary>
    Public Overrides Sub Rollback()
        Me._wrappedTransaction.Rollback()
    End Sub
End Class