﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Data.Common
Imports System.Data.EntityClient
Imports System.Data.Objects

''' <summary>
''' Extension methods for handing wrapped providers.
''' </summary>
Public Module EFProviderWriterExtensionMethods
    ''' <summary>
    ''' Gets the underlying wrapper connection from the <see cref="ObjectContext"/>.
    ''' </summary>
    ''' <typeparam name="TConnection">Connection type.</typeparam>
    ''' <param name="context">The object context.</param>
    ''' <returns>Wrapper connection of a given type.</returns>
    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1004:GenericMethodsShouldProvideTypeParameter", Justification:="Type parameter must be specified explicitly.")>
    <System.Runtime.CompilerServices.Extension()> _
    Public Function UnwrapConnection(Of TConnection As DbConnection)(ByVal context As ObjectContext) As TConnection
        If context Is Nothing Then
            Throw New ArgumentNullException("context")
        End If

        Return context.Connection.UnwrapConnection(Of TConnection)()
    End Function

    ''' <summary>
    ''' Tries to get the underlying wrapper connection from the <see cref="ObjectContext"/>.
    ''' </summary>
    ''' <typeparam name="TConnection">Connection type.</typeparam>
    ''' <param name="context">The object context.</param>
    ''' <param name="result">The result connection.</param>
    ''' <returns>A value of true if the given connection type was found in the provider chain, false otherwise.</returns>
    <System.Runtime.CompilerServices.Extension()> _
    Public Function TryUnwrapConnection(Of TConnection As DbConnection)(ByVal context As ObjectContext, <System.Runtime.InteropServices.Out()> ByRef result As TConnection) As Boolean
        If context Is Nothing Then
            Throw New ArgumentNullException("context")
        End If

        Return context.Connection.TryUnwrapConnection(Of TConnection)(result)
    End Function

    ''' <summary>
    ''' Gets the underlying wrapper connection from the <see cref="DbConnection"/>.
    ''' </summary>
    ''' <typeparam name="TConnection">Connection type.</typeparam>
    ''' <param name="connection">The connection.</param>
    ''' <returns>Wrapper connection of a given type.</returns>
    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1004:GenericMethodsShouldProvideTypeParameter", Justification:="Type parameter must be specified explicitly.")>
    <System.Runtime.CompilerServices.Extension()> _
    Public Function UnwrapConnection(Of TConnection As DbConnection)(ByVal connection As DbConnection) As TConnection
        Dim result As TConnection = Nothing

        If connection.TryUnwrapConnection(result) Then
            Return result
        End If

        Throw New InvalidOperationException("Wrapper provider of type " & GetType(TConnection).FullName & " was not found in the chain.")
    End Function

    ''' <summary>
    ''' Tries to get the underlying wrapper connection from the <see cref="DbConnection"/>.
    ''' </summary>
    ''' <typeparam name="TConnection">Connection type.</typeparam>
    ''' <param name="connection">The connection.</param>
    ''' <param name="result">The result connection.</param>
    ''' <returns>A value of true if the given connection type was found in the provider chain, false otherwise.</returns>
    <System.Runtime.CompilerServices.Extension()> _
    Public Function TryUnwrapConnection(Of TConnection As DbConnection)(ByVal connection As DbConnection, <System.Runtime.InteropServices.Out()> ByRef result As TConnection) As Boolean
        Dim ec As EntityConnection = TryCast(connection, EntityConnection)
        If ec IsNot Nothing Then
            connection = ec.StoreConnection
        End If

        Do While TypeOf connection Is DbConnectionWrapper
            If TypeOf connection Is TConnection Then
                result = CType(connection, TConnection)
                Return True
            End If

            connection = (CType(connection, DbConnectionWrapper)).WrappedConnection
        Loop

        result = Nothing
        Return False
    End Function
End Module