﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports Microsoft.VisualBasic
Imports System
Imports System.Data.Common
Imports System.Data.Common.CommandTrees
Imports System.Globalization
Imports System.Text

''' <summary>
''' Arguments to <see cref="EFTracingConnection.CommandExecuting" />, <see cref="EFTracingConnection.CommandFinished" /> 
''' and <see cref="EFTracingConnection.CommandFailed" /> events.
''' </summary>
Public Class CommandExecutionEventArgs
    Inherits EventArgs
    ''' <summary>
    ''' Initializes a new instance of the CommandExecutionEventArgs class.
    ''' </summary>
    ''' <param name="command">The command.</param>
    ''' <param name="method">The method.</param>
    Friend Sub New(ByVal command As EFTracingCommand, ByVal method As String)
        Me.CommandId = command.CommandID
        Me.Command = command
        Me.CommandTree = command.Definition.CommandTree
        Me.Method = method
    End Sub

    ''' <summary>
    ''' Gets the command ID.
    ''' </summary>
    ''' The command ID.
    Private _CommandId As Integer
    Public Property CommandId() As Integer
        Get
            Return _CommandId
        End Get
        Private Set(ByVal value As Integer)
            _CommandId = value
        End Set
    End Property

    ''' <summary>
    ''' Gets the command object.
    ''' </summary>
    ''' The command.
    Private _Command As DbCommand
    Public Property Command() As DbCommand
        Get
            Return _Command
        End Get
        Private Set(ByVal value As DbCommand)
            _Command = value
        End Set
    End Property

    ''' <summary>
    ''' Gets the command tree.
    ''' </summary>
    ''' The command tree.
    Private _CommandTree As DbCommandTree
    Public Property CommandTree() As DbCommandTree
        Get
            Return _CommandTree
        End Get
        Private Set(ByVal value As DbCommandTree)
            _CommandTree = value
        End Set
    End Property

    ''' <summary>
    ''' Gets the method which caused command execution (ExecuteScalar, ExecuteQuery, ExecuteNonQuery).
    ''' </summary>
    ''' The method name.
    Private _Method As String
    Public Property Method() As String
        Get
            Return _Method
        End Get
        Private Set(ByVal value As String)
            _Method = value
        End Set
    End Property

    ''' <summary>
    ''' Gets the execution status.
    ''' </summary>
    '''Execution status.
    Private _Status As CommandExecutionStatus
    Public Property Status() As CommandExecutionStatus
        Get
            Return _Status
        End Get
        Friend Set(ByVal value As CommandExecutionStatus)
            _Status = value
        End Set
    End Property

    ''' <summary>
    ''' Gets the command result.
    ''' </summary>
    ''' The command result.
    Private _Result As Object
    Public Property Result() As Object
        Get
            Return _Result
        End Get
        Friend Set(ByVal value As Object)
            _Result = value
        End Set
    End Property

    ''' <summary>
    ''' Gets the time it took to execute the command.
    ''' </summary>
    ''' The duration.
    Private _Duration As TimeSpan
    Public Property Duration() As TimeSpan
        Get
            Return _Duration
        End Get
        Friend Set(ByVal value As TimeSpan)
            _Duration = value
        End Set
    End Property

    ''' <summary>
    ''' Returns textual dump of the command suitable for putting in a log file.
    ''' </summary>
    ''' <returns>Textual dump of the command including command text and parameters, 
    ''' suitable for putting in a log file.</returns>
    Public Function ToTraceString() As String
        Dim sb As New StringBuilder()
        sb.Append(Me.Command.CommandText)

        For Each par As DbParameter In Me.Command.Parameters
            sb.AppendFormat(CultureInfo.InvariantCulture, vbCrLf & "-- {0} (dbtype={1}, size={4}, direction={2}) = {3}", par.ParameterName, par.DbType, par.Direction, GetValueText(par.Value), par.Size)
        Next par

        Return sb.ToString()
    End Function

    Private Shared Function GetValueText(ByVal value As Object) As String
        If value Is Nothing OrElse TypeOf value Is DBNull Then
            Return "null"
        ElseIf TypeOf value Is String Then
            Return """" & value & """"
        Else
            Return value.ToString()
        End If
    End Function
End Class