﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Data.Common
Imports System.Data.Objects
Imports System.IO
Imports EFProviderWrapperToolkit

''' <summary>
''' Extension methods for EFTracingProvider.
''' </summary>
Public Module EFTracingExtensionMethods
    ''' <summary>
    ''' Gets the instance of the wrapped <see cref="EFTracingConnection" /> from <see cref="ObjectContext"/>.
    ''' </summary>
    ''' <param name="context">The object context.</param>
    ''' <returns>Instance of <see cref="EFTracingConnection"/>.</returns>
    <System.Runtime.CompilerServices.Extension()> _
    Public Function GetTracingConnection(ByVal context As ObjectContext) As EFTracingConnection
        If context Is Nothing Then
            Throw New ArgumentNullException("context")
        End If

        Return GetTracingConnection(context.Connection)
    End Function

    ''' <summary>
    ''' Gets the instance of the wrapped <see cref="EFTracingConnection" /> from <see cref="DbConnection"/>.
    ''' </summary>
    ''' <param name="connection">The connection object.</param>
    ''' <returns>Instance of <see cref="EFTracingConnection"/>.</returns>
    <System.Runtime.CompilerServices.Extension()> _
    Public Function GetTracingConnection(ByVal connection As DbConnection) As EFTracingConnection
        If connection Is Nothing Then
            Throw New ArgumentNullException("connection")
        End If

        Return connection.UnwrapConnection(Of EFTracingConnection)()
    End Function

    ''' <summary>
    ''' Sets the trace output.
    ''' </summary>
    ''' <param name="context">The context.</param>
    ''' <param name="output">The output text writer.</param>
    <System.Runtime.CompilerServices.Extension()> _
    Public Sub SetTraceOutput(ByVal context As ObjectContext, ByVal output As TextWriter)
        If context Is Nothing Then
            Throw New ArgumentNullException("context")
        End If

        SetTraceOutput(context.Connection, output)
    End Sub

    ''' <summary>
    ''' Sets the trace output.
    ''' </summary>
    ''' <param name="connection">The connection.</param>
    ''' <param name="output">The output text writer.</param>
    <System.Runtime.CompilerServices.Extension()> _
    Public Sub SetTraceOutput(ByVal connection As DbConnection, ByVal output As TextWriter)
        AddHandler connection.UnwrapConnection(Of EFTracingConnection)().CommandExecuting, Sub(sender, e) output.WriteLine(e.ToTraceString())
    End Sub
End Module