﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System
Imports System.Configuration
Imports System.Data
Imports EFProviderWrapperToolkit

''' <summary>
''' Configuration of EFTracingProvider.
''' </summary>
''' <remarks>
''' Default values for properties of this class are obtained by reading configuration file:
''' <para>'EFTracingProvider.logToConsole' parameter supplies default value for.</para>
''' <para>'EFTracingProvider.logToFile' parameter supplies default value for .</para>
''' </remarks>
Public NotInheritable Class EFTracingProviderConfiguration
    ''' <summary>
    ''' Initializes static members of the EFTracingProviderConfiguration class.
    ''' </summary>
    Private Sub New()
    End Sub
    Shared Sub New()
        DefaultWrappedProvider = ConfigurationManager.AppSettings("EFTracingProvider.wrappedProvider")
        LogToConsole = ConfigurationManager.AppSettings("EFTracingProvider.logToConsole") = "true"
        LogToFile = ConfigurationManager.AppSettings("EFTracingProvider.logToFile")
        If String.IsNullOrEmpty(LogToFile) Then
            LogToFile = Nothing
        End If
    End Sub

    ''' <summary>
    ''' Gets or sets the default wrapped provider.
    ''' </summary>
    ''' <value>The default wrapped provider.</value>
    Public Shared Property DefaultWrappedProvider() As String

    ''' <summary>
    ''' Gets or sets a value indicating whether to log to console for all new connections.
    ''' </summary>
    ''' <value>A value of <c>true</c> if new connections should log all statements to the console; otherwise, <c>false</c>.</value>
    Public Shared Property LogToConsole() As Boolean

    ''' <summary>
    ''' Gets or sets the name of the log file for newly created connections.
    ''' </summary>
    ''' <value>The log to file.</value>
    Public Shared Property LogToFile() As String

    ''' <summary>
    ''' Gets or sets the action to be executed before and after every command.
    ''' </summary>
    ''' <value>The log action.</value>
    Public Shared Property LogAction() As Action(Of CommandExecutionEventArgs)

    ''' <summary>
    ''' Registers the provider factory 
    ''' </summary>
    Public Shared Sub RegisterProvider()
        DbProviderFactoryBase.RegisterProvider("EF Tracing Data Provider", "EFTracingProvider", GetType(EFTracingProviderFactory))
    End Sub
End Class