﻿' Copyright (c) Microsoft Corporation.  All rights reserved.

Imports System.Data.Common
Imports System.Data.Common.CommandTrees
Imports EFProviderWrapperToolkit

''' <summary>
''' Implementation of <see cref="DbProviderServices"/> for EFTracingProvider.
''' </summary>
Friend Class EFTracingProviderServices
    Inherits DbProviderServicesBase
    ''' <summary>
    ''' Initializes static members of the EFTracingProviderServices class.
    ''' </summary>
    Shared Sub New()
        Instance = New EFTracingProviderServices()
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the EFTracingProviderServices class.
    ''' </summary>
    Public Sub New()
    End Sub

    ''' <summary>
    ''' Gets the singleton instance.
    ''' </summary>
    ''' The singleton instance.
    Private Shared privateInstance As EFTracingProviderServices
    Public Shared Property Instance() As EFTracingProviderServices
        Get
            Return privateInstance
        End Get
        Private Set(ByVal value As EFTracingProviderServices)
            privateInstance = value
        End Set
    End Property

    ''' <summary>
    ''' Gets the default name of the wrapped provider.
    ''' </summary>
    ''' <returns>
    ''' Default name of the wrapped provider (to be used when
    ''' provider is not specified in the connction string).
    ''' </returns>
    Protected Overrides ReadOnly Property DefaultWrappedProviderName() As String
        Get
            Return EFTracingProviderConfiguration.DefaultWrappedProvider
        End Get
    End Property

    ''' <summary>
    ''' Gets the provider invariant iname.
    ''' </summary>
    ''' <returns>Provider invariant name.</returns>
    Protected Overrides ReadOnly Property ProviderInvariantName() As String
        Get
            Return "EFTracingProvider"
        End Get
    End Property

    ''' <summary>
    ''' Creates the command definition wrapper.
    ''' </summary>
    ''' <param name="wrappedCommandDefinition">The wrapped command definition.</param>
    ''' <param name="commandTree">The command tree.</param>
    ''' <returns>
    ''' The <see cref="DbCommandDefinitionWrapper"/> object.
    ''' </returns>
    Public Overrides Function CreateCommandDefinitionWrapper(ByVal wrappedCommandDefinition As DbCommandDefinition, ByVal commandTree As DbCommandTree) As DbCommandDefinitionWrapper
        Return New DbCommandDefinitionWrapper(wrappedCommandDefinition, commandTree, Function(cmd, def) New EFTracingCommand(cmd, def))
    End Function
End Class