﻿Imports System
Imports System.IO
Imports EFCachingProvider
Imports EFCachingProvider.Caching
Imports EFProviderWrapperToolkit
Imports EFTracingProvider

Partial Public Class ExtendedNorthwindEntities
    Inherits NorthwindEFEntities
    Private logOutput As TextWriter

    Public Sub New()
        Me.New("name=NorthwindEFEntities")
    End Sub

    Public Sub New(ByVal connectionString As String)
        MyBase.New(EntityConnectionWrapperUtils.CreateEntityConnectionWithWrappers(connectionString, "EFTracingProvider", "EFCachingProvider"))
    End Sub

#Region "Tracing Extensions"

    Private ReadOnly Property TracingConnection() As EFTracingConnection
        Get
            Return Me.UnwrapConnection(Of EFTracingConnection)()
        End Get
    End Property

 Public Custom Event CommandExecuting As EventHandler(Of CommandExecutionEventArgs)
        AddHandler(ByVal value As EventHandler(Of CommandExecutionEventArgs))
            AddHandler Me.TracingConnection.CommandExecuting, value
        End AddHandler
        RemoveHandler(ByVal value As EventHandler(Of CommandExecutionEventArgs))
            RemoveHandler Me.TracingConnection.CommandExecuting, value
        End RemoveHandler
        RaiseEvent(ByVal sender As System.Object, ByVal e As CommandExecutionEventArgs)

        End RaiseEvent
    End Event

    Public Custom Event CommandFinished As EventHandler(Of CommandExecutionEventArgs)
        AddHandler(ByVal value As EventHandler(Of CommandExecutionEventArgs))
            AddHandler Me.TracingConnection.CommandFinished, value
        End AddHandler
        RemoveHandler(ByVal value As EventHandler(Of CommandExecutionEventArgs))
            RemoveHandler Me.TracingConnection.CommandFinished, value
        End RemoveHandler
        RaiseEvent(ByVal sender As System.Object, ByVal e As CommandExecutionEventArgs)
            'RaiseEvent Me.TracingConnection.CommandFinished
        End RaiseEvent
    End Event

    Public Custom Event CommandFailed As EventHandler(Of CommandExecutionEventArgs)
        AddHandler(ByVal value As EventHandler(Of CommandExecutionEventArgs))
            AddHandler Me.TracingConnection.CommandFailed, value
        End AddHandler
        RemoveHandler(ByVal value As EventHandler(Of CommandExecutionEventArgs))
            RemoveHandler Me.TracingConnection.CommandFailed, value
        End RemoveHandler
        RaiseEvent(ByVal sender As System.Object, ByVal e As CommandExecutionEventArgs)
            ' RaiseEvent CommandFailed(sender, e)
        End RaiseEvent
    End Event

    Private Sub AppendToLog(ByVal sender As Object, ByVal e As CommandExecutionEventArgs)
        If Me.logOutput IsNot Nothing Then
            Me.logOutput.WriteLine(e.ToTraceString().TrimEnd())
            Me.logOutput.WriteLine()
        End If
    End Sub

    Public Property Log() As TextWriter
        Get
            Return Me.logOutput
        End Get
        Set(ByVal value As TextWriter)
            If (Me.logOutput IsNot Nothing) <> (value IsNot Nothing) Then
                If value Is Nothing Then
                    RemoveHandler CommandExecuting, AddressOf AppendToLog
                Else
                    AddHandler CommandExecuting, AddressOf AppendToLog
                End If
            End If

            Me.logOutput = value
        End Set
    End Property


#End Region

#Region "Caching Extensions"

    Private ReadOnly Property CachingConnection() As EFCachingConnection
        Get
            Return Me.UnwrapConnection(Of EFCachingConnection)()
        End Get
    End Property

    Public Property Cache() As ICache
        Get
            Return CachingConnection.Cache
        End Get
        Set(ByVal value As ICache)
            CachingConnection.Cache = value
        End Set
    End Property

    Public Property CachingPolicy() As CachingPolicy
        Get
            Return CachingConnection.CachingPolicy
        End Get
        Set(ByVal value As CachingPolicy)
            CachingConnection.CachingPolicy = value
        End Set
    End Property

#End Region
End Class
